using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Globalization;
using Excel = Microsoft.Office.Interop.Excel;

namespace ExcelTimesheet
{
	/// <summary>
	///   Add-in Express Excel Worksheet Module
	/// </summary>
	[GuidAttribute("6677F557-CADD-4ef7-AFCF-EBD313507F8D"), ProgId("ExcelTimesheet2005.TimesheetModule")]
	public class TimesheetModule : AddinExpress.MSO.ADXExcelSheetModule, System.Windows.Forms.IWin32Window
	{
		public TimesheetModule()
		{
			InitializeComponent();
		}

		private AddinExpress.MSO.ADXMSFormsComboBox adxEmployeeBox;
		private AddinExpress.MSO.ADXMSFormsComboBox adxApprovedByBox;
		private AddinExpress.MSO.ADXMSFormsCommandButton adxPeriodFromBtn;
		private AddinExpress.MSO.ADXMSFormsCommandButton adxPeriodToBtn;
		private AddinExpress.MSO.ADXMSFormsCommandButton adxImportBtn;
		private AddinExpress.MSO.ADXCommandBar adxCommandBar1;
		private AddinExpress.MSO.ADXCommandBarButton adxClearTimesheet;

		#region Component Designer generated code
		/// <summary>
		/// Required by designer
		/// </summary>
		private System.ComponentModel.IContainer components;

		/// <summary>
		/// Required by designer support - do not modify
		/// the following method
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            this.adxEmployeeBox = new AddinExpress.MSO.ADXMSFormsComboBox(this.components);
            this.adxApprovedByBox = new AddinExpress.MSO.ADXMSFormsComboBox(this.components);
            this.adxPeriodFromBtn = new AddinExpress.MSO.ADXMSFormsCommandButton(this.components);
            this.adxPeriodToBtn = new AddinExpress.MSO.ADXMSFormsCommandButton(this.components);
            this.adxImportBtn = new AddinExpress.MSO.ADXMSFormsCommandButton(this.components);
            this.adxCommandBar1 = new AddinExpress.MSO.ADXCommandBar(this.components);
            this.adxClearTimesheet = new AddinExpress.MSO.ADXCommandBarButton(this.components);
            // 
            // adxEmployeeBox
            // 
            this.adxEmployeeBox.ControlName = "EmployeeBox";
            this.adxEmployeeBox.Click += new System.EventHandler(this.adxEmployeeBox_Click);
            this.adxEmployeeBox.Connect += new System.EventHandler(this.adxEmployeeBox_Connect);
            // 
            // adxApprovedByBox
            // 
            this.adxApprovedByBox.ControlName = "ApprovedByBox";
            this.adxApprovedByBox.Click += new System.EventHandler(this.adxApprovedByBox_Click);
            this.adxApprovedByBox.Connect += new System.EventHandler(this.adxApprovedByBox_Connect);
            // 
            // adxPeriodFromBtn
            // 
            this.adxPeriodFromBtn.ControlName = "PeriodFromBtn";
            this.adxPeriodFromBtn.Click += new System.EventHandler(this.adxPeriodFromBtn_Click);
            // 
            // adxPeriodToBtn
            // 
            this.adxPeriodToBtn.ControlName = "PeriodToBtn";
            this.adxPeriodToBtn.Click += new System.EventHandler(this.adxPeriodToBtn_Click);
            // 
            // adxImportBtn
            // 
            this.adxImportBtn.ControlName = "ImportBtn";
            this.adxImportBtn.Click += new System.EventHandler(this.adxImportBtn_Click);
            // 
            // adxCommandBar1
            // 
            this.adxCommandBar1.CommandBarName = "Excel Timesheet Bar";
            this.adxCommandBar1.CommandBarTag = "c5203094-114e-42f5-af0e-93441442b896";
            this.adxCommandBar1.Controls.Add(this.adxClearTimesheet);
            this.adxCommandBar1.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaExcel;
            this.adxCommandBar1.UpdateCounter = 9;
            // 
            // adxClearTimesheet
            // 
            this.adxClearTimesheet.Caption = "Clear Timesheet";
            this.adxClearTimesheet.ControlTag = "8b0cb1ee-366b-486b-9d7e-fdef6aef0157";
            this.adxClearTimesheet.ImageTransparentColor = System.Drawing.Color.Transparent;
            this.adxClearTimesheet.UpdateCounter = 11;
            this.adxClearTimesheet.Click += new AddinExpress.MSO.ADXClick_EventHandler(this.adxClearTimesheet_Click);
            // 
            // TimesheetModule
            // 
            this.Description = "ADX Excel Worksheet Module Example";
            this.ModuleName = "TimesheetModule";
            this.PropertyId = "_ADX_ExcelTimesheet";
            this.PropertyValue = "Created for the ExcelTimesheet project.";
            this.Worksheet = "Timesheet";
            this.Connect += new System.EventHandler(this.TimecardModule_Connect);
            this.Activate += new System.EventHandler(this.TimesheetModule_Activate);
            this.Deactivate += new System.EventHandler(this.TimesheetModule_Deactivate);

		}
		#endregion

		#region ADX automatic code

		// Required by Add-in Express - do not modify
		// the methods within this region

		public override System.ComponentModel.IContainer GetContainer()
		{
			if (components == null)
				components = new System.ComponentModel.Container();
			return components;
		}

		#endregion

		private string[] Employees = new string[]{
													 "Faas, Matt",
													 "Parker, Daniel",
													 "Smith, John" };

		private string[] Positions = new string[]{
													 "Sales Manager",
													 "Supplier",
													 "Office worker" };

		private string[] Departments = new string[]{
													 "Marketing department",
													 "Bespoke department",
													 "Requisitioning department" };

		private string[] Numbers = new string[]{
													 "23486",
													 "20465",
													 "54789" };

		public System.IntPtr Handle
		{
			get
			{
				return IntPtr.Zero;
			}
		}

		private void adxEmployeeBox_Connect(object sender, System.EventArgs e)
		{
			try
			{
				AddinExpress.MSO.ADXMSFormsComboBox cmb;
				cmb = MSFControlByName("EmployeeBox") as AddinExpress.MSO.ADXMSFormsComboBox;
				if (cmb.ListCount == 0)
				{
					cmb.AddItem(Employees[0]);
					cmb.AddItem(Employees[1]);
					cmb.AddItem(Employees[2]);
				}
			}
			catch (Exception err)
			{
				MessageBox.Show(err.Message, err.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void adxApprovedByBox_Connect(object sender, System.EventArgs e)
		{
			try
			{
				AddinExpress.MSO.ADXMSFormsComboBox cmb;
				cmb = MSFControlByName("ApprovedByBox") as AddinExpress.MSO.ADXMSFormsComboBox;
				if (cmb.ListCount == 0)
				{
					cmb.AddItem("Harrison, George, director");
				}
			}
			catch (Exception err)
			{
				MessageBox.Show(err.Message, err.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void adxEmployeeBox_Click(object sender, System.EventArgs e)
		{
			try
			{
				Excel.Range r = null;
				try
				{
					r = GetRange("E10", System.Type.Missing) as Excel.Range;
					if (r != null)
						r.Value = Employees[adxEmployeeBox.ListIndex];
				}
				finally
				{
					if (r != null)
						Marshal.ReleaseComObject(r);
					r = null;
				}

				try
				{
					r = GetRange("E11", System.Type.Missing) as Excel.Range;
					if (r != null)
						r.Value = Positions[adxEmployeeBox.ListIndex];
				}
				finally
				{
					if (r != null)
						Marshal.ReleaseComObject(r);
					r = null;
				}

				try
				{
					r = GetRange("E12", System.Type.Missing) as Excel.Range;
					if (r != null)
						r.Value = Departments[adxEmployeeBox.ListIndex];
				}
				finally
				{
					if (r != null)
						Marshal.ReleaseComObject(r);
					r = null;
				}

				try
				{
					r = GetRange("I10", System.Type.Missing) as Excel.Range;
					if (r != null)
						r.Value = Numbers[adxEmployeeBox.ListIndex];
				}
				finally
				{
					if (r != null)
						Marshal.ReleaseComObject(r);
					r = null;
				}
				adxImportBtn.Enabled = true;
				adxEmployeeBox.Enabled = true;
				adxApprovedByBox.Enabled = true;
				adxPeriodFromBtn.Enabled = true;
				adxPeriodToBtn.Enabled = true;
			}
			catch (Exception err)
			{
				MessageBox.Show(err.Message, err.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private Excel.Application excelApp = null;

		private void TimecardModule_Connect(object sender, System.EventArgs e)
		{
			excelApp = MainModule.HostApplication as Excel.Application;
		}

		private void adxPeriodFromBtn_Click(object sender, System.EventArgs e)
		{
			ShowCalendar("E16");
		}

		private void adxPeriodToBtn_Click(object sender, System.EventArgs e)
		{
			ShowCalendar("G16");
		}

		private void ShowCalendar(string cell)
		{
			Excel.Range r = null;
			CalendarForm f = null;
			try
			{
				f = new CalendarForm();
				if (f.ShowDialog(this) == DialogResult.OK)
				{
					r = GetRange(cell, System.Type.Missing) as Excel.Range;
					if (r != null)
						r.Value = f.Calendar.SelectionStart.ToShortDateString();
				}
			}
			catch (Exception err)
			{
				MessageBox.Show(err.Message, err.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			finally
			{
				if (r != null)
					Marshal.ReleaseComObject(r);
				if (f != null)
					f.Dispose();
			}
		}

		private void adxApprovedByBox_Click(object sender, System.EventArgs e)
		{
			try
			{
				Excel.Range r = null;
				try
				{
					r = GetRange("M16", System.Type.Missing) as Excel.Range;
					if (r != null)
						r.Value = "Harrison, George, director";
				}
				finally
				{
					if (r != null)
						Marshal.ReleaseComObject(r);
				}
			}
			catch (Exception err)
			{
				MessageBox.Show(err.Message, err.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ClearRanges()
		{
			Excel.Range r = null;
			try
			{
				r = GetRange("D19", "P35") as Excel.Range;
				if (r != null)
					r.ClearContents();
			}
			finally
			{
				if (r != null)
					Marshal.ReleaseComObject(r);
				r = null;
			}
		}

		private void adxImportBtn_Click(object sender, System.EventArgs e)
		{
			Excel.Range r = null;
			System.Random rnd = new System.Random();
			int rows = rnd.Next(1, 17);
			string[] letters = new string[] { "J", "K", "L", "M", "N", "O", "P" };

			try
			{
				ClearRanges();
				for (int i = 1; i <= rows; i++)
				{
					try
					{
						r = GetRange("D" + ((int)(18 + i)).ToString(), System.Type.Missing) as Excel.Range;
						if (r != null)
							r.Value = i.ToString() + ".";
					}
					finally
					{
						if (r != null)
							Marshal.ReleaseComObject(r);
						r = null;
					}
					try
					{
						r = GetRange("E" + ((int)(18 + i)).ToString(), System.Type.Missing) as Excel.Range;
						if (r != null)
							r.Value = "Task " + i.ToString() + ".";
					}
					finally
					{
						if (r != null)
							Marshal.ReleaseComObject(r);
						r = null;
					}

					int days = rnd.Next(1, 7);
					for (int d = 0; d < days; d++)
					{
						try
						{
							r = GetRange(letters[d] + ((int)(18 + i)).ToString(), System.Type.Missing) as Excel.Range;
							if (r != null)
								r.Value = rnd.Next(1, 16);
						}
						finally
						{
							if (r != null)
								Marshal.ReleaseComObject(r);
							r = null;
						}
					}
				}
			}
			catch (Exception err)
			{
				MessageBox.Show(err.Message, err.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		public void OnWindowActivate(object workbook)
		{
			if (IsConnected)
			{
				Excel._Workbook book = null;
				try
				{
					book = this.Parent as Excel._Workbook;
					string parentBookFullName = (string)book.GetType().InvokeMember(
						"FullName", System.Reflection.BindingFlags.GetProperty, null, book, null, CultureInfo.CurrentUICulture);
					string workbookFullName = (string)workbook.GetType().InvokeMember(
						"FullName", System.Reflection.BindingFlags.GetProperty, null, workbook, null, CultureInfo.CurrentUICulture);
					if (String.Compare(parentBookFullName, workbookFullName, true) == 0)
					{
						Excel._Worksheet sheet = null;
						try
						{
							sheet = book.ActiveSheet as Excel._Worksheet;
							if (sheet != null)
							{
								if (String.Compare(sheet.Name, Worksheet, true) == 0)
									adxClearTimesheet.Enabled = true;
							}
						}
						finally
						{
							Marshal.ReleaseComObject(sheet);
						}
					}
				}
				catch (Exception err)
				{
					MessageBox.Show(err.Message, err.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
				finally
				{
					if (book != null)
						Marshal.ReleaseComObject(book);
				}
			}
		}

		private void TimesheetModule_Activate(object sender, System.EventArgs e)
		{
			if (IsConnected)
				adxClearTimesheet.Enabled = true;
		}

		private void TimesheetModule_Deactivate(object sender, System.EventArgs e)
		{
			adxClearTimesheet.Enabled = false;
		}

		private void ClearRange(object cell1, object cell2)
		{
			Excel.Range r = null;
			try
			{
				r = GetRange(cell1, cell2) as Excel.Range;
				if (r != null)
					r.ClearContents();
			}
			finally
			{
				if (r != null)
					Marshal.ReleaseComObject(r);
				r = null;
			}
		}

		private void adxClearTimesheet_Click(object sender)
		{
			try
			{
				ClearRanges();
				ClearRange("E16", System.Type.Missing);
				ClearRange("G16", "H16");
				ClearRange("M16", "Q16");
				ClearRange("E10", "F12");
				ClearRange("I10", System.Type.Missing);
				ClearRange("M9", "Q12");
				adxApprovedByBox.ListIndex = -1;
				adxEmployeeBox.ListIndex = -1;
			}
			catch (Exception err)
			{
				MessageBox.Show(err.Message, err.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
	}
}
